@@ -54,18 +54,6 @@ OC.L10N.register(
     "{displayName} (team)" : "{displayName} (فريق)",
     "Add group or team" : "إضافة مجموعة أو فريق",
     "{size} used" : "تم استهلاك{size} ",
-    "You can not remove your own read permission." : "لايمكنك إزالة إذن القراءة الخاص بك.",
-    "Group folders" : "مجلدات المجموعة",
-    "Admin configured folders shared with everyone in a group" : "مجلدات المجموعة مشتركة بين كل أعضاء المجموعة و يديرها المدير",
-    "Admin configured folders shared with everyone in a group.\n\nFolders can be configured from *Group folders* in the admin settings.\n\nAfter a folder is created, the admin can give access to the folder to one or more groups, control their write/sharing permissions and assign a quota for the folder." : "المجلدات التي يقوم المدير بتكوينها تتم مشاركتها مع جميع أعضاء المجموعة. يمكن تكوين المجلدات من *مجلدات المجموعة Group folders* في إعدادات المدير. بعد إنشاء المجلد، يمكن للمدير  منح حق الوصول إليه لمجموعة واحدة أو أكثر، و كذلك التحكم في أذونات الكتابة و المشاركة الخاصة بهم وتعيين حصة تخزينية للمجلد.",
-    "Group folder" : "مجلد مجموعة",
-    "Open group folder" : "فتح مجلد المجموعة",
-    "List of group folders." : "قائمة مجلدات المجموعة",
-    "No group folders yet" : "لا توجد أي مجلدات مجموعة حتى الآن",
-    "Group folders will show up here" : "مجلدات المجموعة ستظهر هنا",
-    "Group folder admin delegation" : "تفويض صلاحيات مدير مجلد مجموعة",
-    "They will have access to all group folders." : "سوف يمكنهم الوصول إلى جميع مجلدات المجموعة",
-    "They will only have access to group folders for which they have advanced permissions." : "سوف يمكنهم الوصول فقط إلى مجلدات المجموعة التي لديهم أذونات وصول متقدمة إليها.",
     "Admin configured folders shared with everyone in a team" : "مُجلّدات كوّنها المُشرف لتكون مشتركة بين جميع أعضاء الفريق ",
     "Admin configured folders shared with everyone in a team.\n\nFolders can be configured from *Team folders* in the admin settings.\n\nAfter a folder is created, the admin can give access to the folder to one or more teams, control their write/sharing permissions and assign a quota for the folder." : "قام المسؤول بتكوين المجلدات المشتركة بين جميع أعضاء الفريق. يمكن تكوين المجلدات من *مجلدات الفريق\"  Team Folders في إعدادات المُشرف. بعد إنشاء المجلد، يمكن للمسؤول منح حق الوصول إلى المجلد لفريق واحد أو أكثر، والتحكم في أذونات الكتابة أو المشاركة الخاصة بهم وتعيين حصة تخزينية للمُجلّد.",
     "Admin configured folders shared with everyone in a team.\n\nFolders can be configured from *Team folders* in the admin settings.\n\nAfter a folder is created, the admin can give access to the folder to one or more teams, control their write/sharing permissions and assign a quota for the folder.\nAs of Hub 10/Nextcloud 31, the admin needs to be a part of the team to be able to assign it a Teamfolder." : "مُجلَّدات تمَّ تكوينها بواسطة المشرف والذي قام بمشاركتها مع جميع أعضاء الفريق. \n\nيمكن تكوين المجلدات من *مجلدات الفريق Team folders* في إعدادات الإدارة. \n\nبعد إنشاء مجلد، يمكن للمسؤول منح حق الوصول إليه لفريق واحد أو أكثر، والتحكم في أذونات الكتابة/المشاركة الخاصة بهم وتعيين حصة تخزينية للمجلد. \n\nاعتباراً من الاصدار العاشر من \"نكست كلاود هَب\" Hub 10/Nextcloud 31، يجب أن يكون المسؤول جزءاً من الفريق حتى يتمكن من تعيين مجلد عامٍّ له."
