@@ -16,7 +16,6 @@ OC.L10N.register(
     "Previous" : "Anterior",
     "User" : "Usuario",
     "None" : "Ninguno",
-    "{size} used" : "{size} usado",
-    "Group folders" : "Carpetas de grupo"
+    "{size} used" : "{size} usado"
 },
 "nplurals=3; plural=n == 1 ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
