@@ -32,7 +32,6 @@ OC.L10N.register(
     "User" : "Utilizador",
     "Unknown" : "Desconhecido",
     "None" : "Nenhum",
-    "{size} used" : "{size} utilizado",
-    "Group folders" : "Pastas de grupo"
+    "{size} used" : "{size} utilizado"
 },
 "nplurals=3; plural=(n == 0 || n == 1) ? 0 : n != 0 && n % 1000000 == 0 ? 1 : 2;");
