@@ -1,4 +1,6 @@
 <?php
+
+declare (strict_types=1);
 /**
  * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
  * SPDX-License-Identifier: AGPL-3.0-or-later
@@ -7,6 +9,7 @@
 namespace OCA\GroupFolders\Trash;
 
 use OC\Encryption\Exceptions\DecryptionFailedException;
+use OC\Files\ObjectStore\ObjectStoreStorage;
 use OC\Files\Storage\Wrapper\Encryption;
 use OC\Files\Storage\Wrapper\Jail;
 use OCA\Files_Trashbin\Expiration;
@@ -14,12 +17,13 @@ use OCA\Files_Trashbin\Storage;
 use OCA\Files_Trashbin\Trash\ITrashBackend;
 use OCA\Files_Trashbin\Trash\ITrashItem;
 use OCA\GroupFolders\ACL\ACLManagerFactory;
+use OCA\GroupFolders\Folder\FolderDefinitionWithPermissions;
 use OCA\GroupFolders\Folder\FolderManager;
+use OCA\GroupFolders\Folder\FolderWithMappingsAndCache;
 use OCA\GroupFolders\Mount\GroupFolderStorage;
 use OCA\GroupFolders\Mount\MountProvider;
 use OCA\GroupFolders\Versions\VersionsBackend;
 use OCP\Constants;
-use OCP\Files\FileInfo;
 use OCP\Files\Folder;
 use OCP\Files\IRootFolder;
 use OCP\Files\Mount\IMountManager;
@@ -34,24 +38,20 @@ use OCP\IUserManager;
 use OCP\IUserSession;
 use Psr\Log\LoggerInterface;
 
-/**
- * @psalm-import-type InternalFolder from FolderManager
- */
 class TrashBackend implements ITrashBackend {
 	private ?VersionsBackend $versionsBackend = null;
 
 	public function __construct(
-		private FolderManager $folderManager,
-		private TrashManager $trashManager,
-		private Folder $appFolder,
-		private MountProvider $mountProvider,
-		private ACLManagerFactory $aclManagerFactory,
-		private IRootFolder $rootFolder,
-		private LoggerInterface $logger,
-		private IUserManager $userManager,
-		private IUserSession $userSession,
-		private IMountManager $mountManager,
-		private IStorageFactory $storageFactory,
+		private readonly FolderManager $folderManager,
+		private readonly TrashManager $trashManager,
+		private readonly ACLManagerFactory $aclManagerFactory,
+		private readonly IRootFolder $rootFolder,
+		private readonly LoggerInterface $logger,
+		private readonly IUserManager $userManager,
+		private readonly IUserSession $userSession,
+		private readonly MountProvider $mountProvider,
+		private readonly IMountManager $mountManager,
+		private readonly IStorageFactory $storageFactory,
 	) {
 	}
 
@@ -82,14 +82,10 @@ class TrashBackend implements ITrashBackend {
 		}
 
 		$content = $folderNode->getDirectoryListing();
-		$this->aclManagerFactory->getACLManager($user)->preloadRulesForFolder($folder->getPath());
+		$this->aclManagerFactory->getACLManager($user)->preloadRulesForFolder($folder->getGroupFolderStorageId(), $folder->getId());
 
 		return array_values(array_filter(array_map(function (Node $node) use ($folder, $user): ?GroupTrashItem {
-			if (!$this->userHasAccessToPath($user, $this->getUnJailedPath($node))) {
-				return null;
-			}
-
-			return new GroupTrashItem(
+			$item = new GroupTrashItem(
 				$this,
 				$folder->getInternalOriginalLocation() . '/' . $node->getName(),
 				$folder->getDeletedTime(),
@@ -98,7 +94,14 @@ class TrashBackend implements ITrashBackend {
 				$user,
 				$folder->getGroupFolderMountPoint(),
 				$folder->getDeletedBy(),
+				$folder->folder,
 			);
+
+			if (!$this->userHasAccessToItem($item)) {
+				return null;
+			}
+
+			return $item;
 		}, $content)));
 	}
 
@@ -112,17 +115,17 @@ class TrashBackend implements ITrashBackend {
 
 		$user = $item->getUser();
 		$userFolder = $this->rootFolder->getUserFolder($user->getUID());
-		[, $folderId] = explode('/', $item->getTrashPath());
+		$folderId = $item->folder->id;
 		$node = $this->getNodeForTrashItem($user, $item);
 		if ($node === null) {
 			throw new NotFoundException();
 		}
 
-		if (!$this->userHasAccessToPath($item->getUser(), $item->getPath(), Constants::PERMISSION_UPDATE)) {
+		if (!$this->userHasAccessToItem($item, Constants::PERMISSION_UPDATE)) {
 			throw new NotPermittedException();
 		}
 
-		$folderPermissions = $this->folderManager->getFolderPermissionsForUser($item->getUser(), (int)$folderId);
+		$folderPermissions = $this->folderManager->getFolderPermissionsForUser($item->getUser(), $folderId);
 		if (($folderPermissions & Constants::PERMISSION_UPDATE) !== Constants::PERMISSION_UPDATE) {
 			throw new NotPermittedException();
 		}
@@ -172,7 +175,7 @@ class TrashBackend implements ITrashBackend {
 		try {
 			$targetStorage->moveFromStorage($trashStorage, $trashLocation, $targetLocation);
 			$targetStorage->getUpdater()->renameFromStorage($trashStorage, $trashLocation, $targetLocation);
-		} catch (DecryptionFailedException $e) {
+		} catch (DecryptionFailedException) {
 			// Before https://github.com/nextcloud/groupfolders/pull/3425 the key would be in the wrong place, leading to the decryption failure.
 			// for those we fall back to the old restore behavior
 			[$unwrappedTargetStorage, $unwrappedTargetLocation] = $this->unwrapJails($targetStorage, $targetLocation);
@@ -180,14 +183,14 @@ class TrashBackend implements ITrashBackend {
 			$unwrappedTargetStorage->moveFromStorage($unwrappedTrashStorage, $unwrappedTrashLocation, $unwrappedTargetLocation);
 			$unwrappedTargetStorage->getUpdater()->renameFromStorage($unwrappedTrashStorage, $unwrappedTrashLocation, $unwrappedTargetLocation);
 		}
-		$this->trashManager->removeItem((int)$folderId, $item->getName(), $item->getDeletedTime());
+		$this->trashManager->removeItem($folderId, $item->getName(), $item->getDeletedTime());
 		\OCP\Util::emitHook(
 			'\OCA\Files_Trashbin\Trashbin',
 			'post_restore',
 			[
 				'filePath' => '/' . $item->getGroupFolderMountPoint() . '/' . $originalLocation,
 				'trashPath' => $item->getPath(),
-			]
+			],
 		);
 	}
 
@@ -213,17 +216,17 @@ class TrashBackend implements ITrashBackend {
 		}
 
 		$user = $item->getUser();
-		[, $folderId] = explode('/', $item->getTrashPath());
+		$folderId = $item->folder->id;
 		$node = $this->getNodeForTrashItem($user, $item);
 		if ($node === null) {
 			throw new NotFoundException();
 		}
 
-		if (!$this->userHasAccessToPath($item->getUser(), $item->getPath(), Constants::PERMISSION_DELETE)) {
+		if (!$this->userHasAccessToItem($item, Constants::PERMISSION_DELETE)) {
 			throw new NotPermittedException();
 		}
 
-		$folderPermissions = $this->folderManager->getFolderPermissionsForUser($item->getUser(), (int)$folderId);
+		$folderPermissions = $this->folderManager->getFolderPermissionsForUser($item->getUser(), $folderId);
 		if (($folderPermissions & Constants::PERMISSION_DELETE) !== Constants::PERMISSION_DELETE) {
 			throw new NotPermittedException();
 		}
@@ -234,7 +237,7 @@ class TrashBackend implements ITrashBackend {
 
 		$node->getStorage()->getCache()->remove($node->getInternalPath());
 		if ($item->isRootItem()) {
-			$this->trashManager->removeItem((int)$folderId, $item->getName(), $item->getDeletedTime());
+			$this->trashManager->removeItem($folderId, $item->getName(), $item->getDeletedTime());
 		}
 	}
 
@@ -243,14 +246,10 @@ class TrashBackend implements ITrashBackend {
 			/** @var GroupFolderStorage $storage */
 			$name = basename($internalPath);
 			$fileEntry = $storage->getCache()->get($internalPath);
+			$folder = $storage->getFolder();
 			$folderId = $storage->getFolderId();
-			$user = $this->userSession->getUser();
 
-			$owner = $storage->getUser();
-
-			$this->setupTrashFolder($folderId, $owner);
-
-			$trashFolder = $this->rootFolder->get('/' . $owner->getUID() . '/files_trashbin/groupfolders/' . $folderId);
+			$trashFolder = $this->setupTrashFolder($folder, $storage->getUser());
 			$trashStorage = $trashFolder->getStorage();
 			$time = time();
 			$trashName = $name . '.d' . $time;
@@ -267,7 +266,8 @@ class TrashBackend implements ITrashBackend {
 					$originalLocation = $storage->getWrapperStorage()->getUnjailedPath($originalLocation);
 				}
 
-				$this->trashManager->addTrashItem($folderId, $name, $time, $originalLocation, $fileEntry->getId(), $user?->getUID() ?? '');
+				$deletedBy = $this->userSession->getUser();
+				$this->trashManager->addTrashItem($folderId, $name, $time, $originalLocation, $fileEntry->getId(), $deletedBy?->getUID() ?? '');
 
 				// some storage backends (object/encryption) can either already move the cache item or cause the target to be scanned
 				// so we only conditionally do the cache move here
@@ -326,33 +326,59 @@ class TrashBackend implements ITrashBackend {
 	}
 
 	private function userHasAccessToFolder(IUser $user, int $folderId): bool {
-		$folders = $this->folderManager->getFoldersForUser($user);
-		$folderIds = array_map(fn (array $folder): int => $folder['folder_id'], $folders);
+		return $this->folderManager->getFoldersForUser($user, $folderId) !== [];
+	}
 
-		return in_array($folderId, $folderIds);
+	private function userHasAccessToItem(
+		GroupTrashItem $item,
+		int $permission = Constants::PERMISSION_READ,
+		string $pathInsideItem = '',
+	): bool {
+		try {
+			$aclManager = $this->aclManagerFactory->getACLManager($item->getUser());
+			$trashPath = $this->getUnJailedPath($item->getTrashNode()) . $pathInsideItem;
+			$activePermissions = $aclManager->getACLPermissionsForPath($item->getGroupTrashFolderStorageId(), $trashPath);
+			$originalPath = $item->folder->rootCacheEntry->getPath() . '/' . $item->getInternalOriginalLocation() . $pathInsideItem;
+			$originalLocationPermissions = $aclManager->getACLPermissionsForPath($item->getGroupFolderStorageId(), $originalPath);
+		} catch (\Exception $e) {
+			$this->logger->warning("Failed to get permissions for {$item->getPath()}", ['exception' => $e]);
+			return false;
+		}
+
+		return (bool)($activePermissions & $permission & $originalLocationPermissions);
 	}
 
 	private function userHasAccessToPath(
+		int $storageId,
 		IUser $user,
 		string $path,
 		int $permission = Constants::PERMISSION_READ,
 	): bool {
-		$activePermissions = $this->aclManagerFactory->getACLManager($user)
-			->getACLPermissionsForPath($path);
+		try {
+			$activePermissions = $this->aclManagerFactory->getACLManager($user)
+				->getACLPermissionsForPath($storageId, $path);
+		} catch (\Exception $e) {
+			$this->logger->warning("Failed to get permissions for {$path}", ['exception' => $e]);
+			return false;
+		}
 
 		return (bool)($activePermissions & $permission);
 	}
 
 	private function getNodeForTrashItem(IUser $user, ITrashItem $trashItem): ?Node {
-		[, $folderId, $path] = explode('/', $trashItem->getTrashPath(), 3);
-		$folderId = (int)$folderId;
-		$folders = $this->folderManager->getFoldersForUser($user);
+		if (!($trashItem instanceof GroupTrashItem)) {
+			throw new \LogicException('Trying to remove normal trash item in Team folder trash backend');
+		}
+
+		$folderId = $trashItem->folder->id;
+		$path = $trashItem->getFullInternalPath();
+		$folders = $this->folderManager->getFoldersForUser($user, $folderId);
 		foreach ($folders as $groupFolder) {
-			if ($groupFolder['folder_id'] === $folderId) {
-				$trashRoot = $this->rootFolder->get('/' . $user->getUID() . '/files_trashbin/groupfolders/' . $folderId);
+			if ($groupFolder->id === $folderId) {
+				$trashRoot = $this->setupTrashFolder($groupFolder, $user);
 				try {
 					$node = $trashRoot->get($path);
-					if (!$this->userHasAccessToPath($user, $trashItem->getPath())) {
+					if (!$this->userHasAccessToItem($trashItem)) {
 						return null;
 					}
 
@@ -366,44 +392,24 @@ class TrashBackend implements ITrashBackend {
 		return null;
 	}
 
-	private function getTrashRoot(): Folder {
-		try {
-			/** @var Folder $folder */
-			$folder = $this->appFolder->get('trash');
+	private function setupTrashFolder(FolderDefinitionWithPermissions $folder, ?IUser $user = null): Folder {
+		$folderId = $folder->id;
 
-			return $folder;
-		} catch (NotFoundException) {
-			return $this->appFolder->newFolder('trash');
-		}
-	}
+		$uid = $user ? $user->getUID() : 'dummy';
 
-	private function setupTrashFolder(int $folderId, ?IUser $user = null): Folder {
-		if ($user) {
-			$mountPoint = '/' . $user->getUID() . '/files_trashbin/groupfolders/' . $folderId;
-			$mount = $this->mountManager->find($mountPoint);
-			if ($mount->getMountPoint() !== $mountPoint) {
-				$trashMount = $this->mountProvider->getTrashMount(
-					$folderId,
-					$mountPoint,
-					FileInfo::SPACE_UNLIMITED,
-					$this->storageFactory,
-					$user,
-				);
-				$this->mountManager->addMount($trashMount);
-			}
+		$mountPoint = '/' . $uid . '/files_trashbin/groupfolders/' . $folderId;
+		$mount = $this->mountManager->find($mountPoint);
+		if ($mount->getMountPoint() !== $mountPoint) {
+			$trashMount = $this->mountProvider->getTrashMount(
+				$folder,
+				$mountPoint,
+				$this->storageFactory,
+				$user,
+			);
+			$this->mountManager->addMount($trashMount);
 		}
 
-		try {
-			/** @var Folder $folder */
-			$folder = $this->appFolder->get('trash/' . $folderId);
-
-			return $folder;
-		} catch (NotFoundException) {
-			/** @var Folder $trashRoot */
-			$trashRoot = $this->appFolder->nodeExists('trash') ? $this->appFolder->get('trash') : $this->appFolder->newFolder('trash');
-
-			return $trashRoot->newFolder((string)$folderId);
-		}
+		return $this->rootFolder->get('/' . $uid . '/files_trashbin/groupfolders/' . $folderId);
 	}
 
 	private function getUnJailedPath(Node $node): string {
@@ -418,142 +424,139 @@ class TrashBackend implements ITrashBackend {
 	}
 
 	/**
-	 * @param list<InternalFolder> $folders
+	 * @param list<FolderDefinitionWithPermissions> $folders
 	 * @return list<ITrashItem>
 	 */
 	private function getTrashForFolders(IUser $user, array $folders): array {
-		$folderIds = array_map(fn (array $folder): int => $folder['folder_id'], $folders);
+		$folderIds = array_map(fn (FolderDefinitionWithPermissions $folder): int => $folder->id, $folders);
 		$rows = $this->trashManager->listTrashForFolders($folderIds);
 		$indexedRows = [];
-		$trashItemsByOriginalLocation = [];
 		foreach ($rows as $row) {
 			$key = $row['folder_id'] . '/' . $row['name'] . '/' . $row['deleted_time'];
 			$indexedRows[$key] = $row;
-			$trashItemsByOriginalLocation[$row['original_location']] = $row;
 		}
 
 		$items = [];
 		foreach ($folders as $folder) {
-			$folderId = $folder['folder_id'];
-			$folderHasAcl = $folder['acl'];
-			$mountPoint = $folder['mount_point'];
-
-			// ensure the trash folder exists
-			$this->setupTrashFolder($folderId, $user);
-
-			$trashFolder = $this->rootFolder->get('/' . $user->getUID() . '/files_trashbin/groupfolders/' . $folderId);
+			// note that we explicitly don't pass the user here, was we need to get all trash items,
+			// not only the trash items we have access to (so we can get their original paths)
+			// we apply acl filtering later to get the correct permissions again
+			$trashFolder = $this->setupTrashFolder($folder);
 			$content = $trashFolder->getDirectoryListing();
-			$userCanManageAcl = $this->folderManager->canManageACL($folderId, $user);
-			$this->aclManagerFactory->getACLManager($user)->preloadRulesForFolder($this->getUnJailedPath($trashFolder));
-			foreach ($content as $item) {
-				/** @var \OC\Files\Node\Node $item */
+			$userCanManageAcl = $this->folderManager->canManageACL($folder->id, $user);
+			$this->aclManagerFactory->getACLManager($user)->preloadRulesForFolder($folder->storageId, $trashFolder->getId());
+
+			$itemsForFolder = array_map(function (Node $item) use ($user, $folder, $indexedRows) {
 				$pathParts = pathinfo($item->getName());
 				$timestamp = (int)substr($pathParts['extension'], 1);
 				$name = $pathParts['filename'];
-				$key = $folderId . '/' . $name . '/' . $timestamp;
+				$key = $folder->id . '/' . $name . '/' . $timestamp;
 
 				$originalLocation = isset($indexedRows[$key]) ? $indexedRows[$key]['original_location'] : '';
 				$deletedBy = isset($indexedRows[$key]) ? $indexedRows[$key]['deleted_by'] : '';
 
-				if ($folderHasAcl) {
-					// if we for any reason lost track of the original location, hide the item for non-managers as a fail-safe
-					if ($originalLocation === '' && !$userCanManageAcl) {
-						continue;
-					}
-
-					if (!$this->userHasAccessToPath($user, $this->getUnJailedPath($item))) {
-						continue;
-					}
-
-					// if a parent of the original location has also been deleted, we also need to check it based on the now-deleted parent path
-					foreach ($this->getParentOriginalPaths($originalLocation, $trashItemsByOriginalLocation) as $parentOriginalPath) {
-						$parentTrashItem = $trashItemsByOriginalLocation[$parentOriginalPath];
-						$relativePath = substr($originalLocation, strlen($parentOriginalPath));
-						$parentTrashItemPath = "__groupfolders/trash/{$parentTrashItem['folder_id']}/{$parentTrashItem['name']}.d{$parentTrashItem['deleted_time']}";
-						if (!$this->userHasAccessToPath($user, $parentTrashItemPath . $relativePath)) {
-							continue 2;
-						}
-					}
-				}
-
-				$info = $item->getFileInfo();
-				$info['name'] = $name;
-				$items[] = new GroupTrashItem(
+				return new GroupTrashItem(
 					$this,
 					$originalLocation,
 					$timestamp,
-					'/' . $folderId . '/' . $item->getName(),
-					$info,
+					'/' . $folder->id . '/' . $item->getName(),
+					$item,
 					$user,
-					$mountPoint,
+					$folder->mountPoint,
 					$this->userManager->get($deletedBy),
+					$folder,
 				);
+			}, $content);
+			$originalLocations = array_map(fn (GroupTrashItem $item): string => $item->getOriginalLocation(), $itemsForFolder);
+			$itemsByOriginalLocation = array_combine($originalLocations, $itemsForFolder);
+
+			// perform per-item ACL checks if the user doesn't have manage permissions
+			if ($folder->acl && !$userCanManageAcl) {
+				$itemsForFolder = array_filter($itemsForFolder, function (GroupTrashItem $item) use ($itemsByOriginalLocation) {
+					// if we for any reason lost track of the original location, hide the item for non-managers as a fail-safe
+					if ($item->getInternalOriginalLocation() === '') {
+						return false;
+					}
+
+					if (!$this->userHasAccessToItem($item)) {
+						return false;
+					}
+
+					// if a parent of the original location has also been deleted, we also need to check it based on the now-deleted parent path
+					foreach ($this->getDeletedParentOriginalPaths($item->getOriginalLocation(), $itemsByOriginalLocation) as $parentItem) {
+						$pathInsideParentItem = dirname(substr($item->getInternalOriginalLocation(), strlen($parentItem->getInternalOriginalLocation())));
+						if (!$this->userHasAccessToItem($parentItem, Constants::PERMISSION_READ, $pathInsideParentItem)) {
+							return false;
+						}
+					}
+
+					return true;
+				});
 			}
+			$items = array_merge($items, $itemsForFolder);
 		}
 
 		return $items;
 	}
 
-	private function getParentOriginalPaths(string $path, array $trashItemsByOriginalPath): array {
-		$parentPaths = [];
+	/**
+	 * @param array<string, GroupTrashItem> $trashItemsByOriginalPath
+	 * @return list<GroupTrashItem>
+	 */
+	private function getDeletedParentOriginalPaths(string $path, array $trashItemsByOriginalPath): array {
+		$parentItems = [];
 		while ($path !== '') {
 			$path = dirname($path);
 
 			if ($path === '.' || $path === '/') {
 				break;
 			} elseif (isset($trashItemsByOriginalPath[$path])) {
-				$parentPaths[] = $path;
+				$parentItems[] = $trashItemsByOriginalPath[$path];
 			}
 		}
 
-		return $parentPaths;
+		return $parentItems;
 	}
 
 	public function getTrashNodeById(IUser $user, int $fileId): ?Node {
 		try {
-			/** @var Folder $trashFolder */
-			$trashFolder = $this->appFolder->get('trash');
-			$storage = $this->appFolder->getStorage();
-			$path = $storage->getCache()->getPathById($fileId);
-			if (!$path) {
-				return null;
-			}
-
-			$absolutePath = $this->appFolder->getMountPoint()->getMountPoint() . $path;
-			$relativePath = $trashFolder->getRelativePath($absolutePath);
-			[, $folderId, $nameAndTime] = explode('/', $relativePath);
-
-			if ($this->userHasAccessToFolder($user, (int)$folderId) && $this->userHasAccessToPath($user, $absolutePath)) {
-				return $trashFolder->get($relativePath);
-			} else {
-				return null;
+			$folders = $this->folderManager->getFoldersForUser($user);
+			foreach ($folders as $folder) {
+				$trashFolder = $this->setupTrashFolder($folder, $user);
+				if ($path = $trashFolder->getStorage()->getCache()->getPathById($fileId)) {
+					return $trashFolder->get($path);
+				}
 			}
+			return null;
 		} catch (NotFoundException) {
 			return null;
 		}
 	}
 
-	public function cleanTrashFolder(int $folderid): void {
-		$trashFolder = $this->setupTrashFolder($folderid);
+	public function cleanTrashFolder(FolderDefinitionWithPermissions $folder): void {
+		$trashFolder = $this->setupTrashFolder($folder);
 
 		foreach ($trashFolder->getDirectoryListing() as $node) {
 			$node->delete();
 		}
 
-		$this->trashManager->emptyTrashbin($folderid);
+		$this->trashManager->emptyTrashbin($folder->id);
 	}
 
 	public function expire(Expiration $expiration): array {
 		$size = 0;
 		$count = 0;
-		$folders = $this->folderManager->getAllFoldersWithSize($this->rootFolder->getMountPoint()->getNumericStorageId());
+		$folders = $this->folderManager->getAllFoldersWithSize();
+		$folders = array_map(function (FolderWithMappingsAndCache $folder): FolderDefinitionWithPermissions {
+			return FolderDefinitionWithPermissions::fromFolder($folder, $folder->rootCacheEntry, Constants::PERMISSION_ALL);
+		}, $folders);
 		foreach ($folders as $folder) {
-			$folderId = $folder['id'];
+			$folderId = $folder->id;
 			$trashItems = $this->trashManager->listTrashForFolders([$folderId]);
 
 			// calculate size of trash items
 			$sizeInTrash = 0;
-			$trashFolder = $this->setupTrashFolder($folderId);
+			$trashFolder = $this->setupTrashFolder($folder);
 			$nodes = []; // cache
 			foreach ($trashItems as $groupTrashItem) {
 				$nodeName = $groupTrashItem['name'] . '.d' . $groupTrashItem['deleted_time'];
@@ -567,6 +570,8 @@ class TrashBackend implements ITrashBackend {
 				$sizeInTrash += $node->getSize();
 			}
 
+			$size = $folder->rootCacheEntry->getSize();
+
 			foreach ($trashItems as $groupTrashItem) {
 				$nodeName = $groupTrashItem['name'] . '.d' . $groupTrashItem['deleted_time'];
 				if (!isset($nodes[$nodeName])) {
@@ -575,7 +580,7 @@ class TrashBackend implements ITrashBackend {
 
 				$node = $nodes[$nodeName];
 
-				if ($expiration->isExpired($groupTrashItem['deleted_time'], $folder['quota'] > 0 && $folder['quota'] < ($folder['size'] + $sizeInTrash))) {
+				if ($expiration->isExpired($groupTrashItem['deleted_time'], $folder->quota > 0 && $folder->quota < ($size + $sizeInTrash))) {
 					$this->logger->debug('expiring ' . $node->getPath());
 					if ($node->getStorage()->unlink($node->getInternalPath()) === false) {
 						$this->logger->error('Failed to remove item from trashbin: ' . $node->getPath());
@@ -585,11 +590,11 @@ class TrashBackend implements ITrashBackend {
 					// only count up after checking if removal is possible
 					$count += 1;
 					$size += $node->getSize();
-					$folder['size'] -= $node->getSize();
+					$size -= $node->getSize();
 					$node->getStorage()->getCache()->remove($node->getInternalPath());
 					$this->trashManager->removeItem($folderId, $groupTrashItem['name'], $groupTrashItem['deleted_time']);
 					if (!is_null($groupTrashItem['file_id']) && !is_null($this->versionsBackend)) {
-						$this->versionsBackend->deleteAllVersionsForFile($folderId, $groupTrashItem['file_id']);
+						$this->versionsBackend->deleteAllVersionsForFile($folder, $groupTrashItem['file_id']);
 					}
 				} else {
 					$this->logger->debug($node->getPath() . " isn't set to be expired yet, stopping expiry");
@@ -598,25 +603,6 @@ class TrashBackend implements ITrashBackend {
 			}
 		}
 
-		$this->cleanupDeletedFoldersTrash($folders);
-
 		return [$count, $size];
 	}
-
-	/**
-	 * Cleanup trashbin of of groupfolders that have been deleted
-	 */
-	private function cleanupDeletedFoldersTrash(array $existingFolders): void {
-		$trashRoot = $this->getTrashRoot();
-		foreach ($trashRoot->getDirectoryListing() as $trashFolder) {
-			$folderId = $trashFolder->getName();
-			if (is_numeric($folderId)) {
-				$folderId = (int)$folderId;
-				if (!isset($existingFolders[$folderId])) {
-					$this->cleanTrashFolder($folderId);
-					$this->setupTrashFolder($folderId)->delete();
-				}
-			}
-		}
-	}
 }
